/**
 * Contains implementation classes of the library. To simplify
 * the implementation is not hidden in any way: it is possible to extend the 
 * existing classes, or to create custom validators using the existing certificate 
 * checker classes. 
 */
package eu.emi.security.authn.x509.impl;

