/**
 * Contains implementation of local namespace policy rules for trusted certificate authorities 
 * (using Globus and EuGridPMA syntax), which are exclusively used by the
 * {@link eu.emi.security.authn.x509.impl.OpensslCertChainValidator}  
 * <p>
 * Warning: this package contains internal implementation of the library. It is not
 * guaranteed that API of the classes from this package will not change in future releases.
 */
package eu.emi.security.authn.x509.helpers.ns;

