/**
 * Helper classes, implementing SSL/TLS/network related functions. 
 * <p>
 * Warning: this package contains internal implementation of the library. It is not
 * guaranteed that API of the classes from this package will not change in future releases.
 */
package eu.emi.security.authn.x509.helpers.ssl;

