/*
 * Copyright (c) 2016 ICM Uniwersytet Warszawski All rights reserved.
 * See LICENCE.txt file for licensing information.
 */
package eu.emi.security.authn.x509;

/**
 * Marker for fragile tests which should not be run with regular build as depend on 
 * hardware or environment settings.
 * @author K. Benedyczak
 */
public interface RiskyIntegrationTests
{

}
